@@ -12,7 +12,7 @@
  * See the file "license.terms" for information on usage and redistribution
  * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
  *
- * RCS: @(#) $Id: tclEnv.c,v 1.4.6.1 1999/10/30 11:06:00 hobbs Exp $
+ * RCS: @(#) $Id: tclEnv.c,v 1.7.2.2 2001/04/03 22:54:37 hobbs Exp $
  */
 
 #include "tclInt.h"
@@ -33,6 +33,14 @@ static int environSize = 0;	/* Non-zero means that the environ array was
 				 * array is in its original static state. */
 #endif
 
+/*
+ * For MacOS X
+ */
+#if defined(__APPLE__) && defined(__DYNAMIC__)
+#include <crt_externs.h>
+char **environ = NULL;
+#endif
+
 /*
  * Declarations for local procedures defined in this file:
  */
@@ -79,6 +87,13 @@ TclSetupEnv(interp)
     char *p1, *p2;
     int i;
 
+    /*
+     * For MacOS X
+     */
+#if defined(__APPLE__) && defined(__DYNAMIC__)
+    environ = *_NSGetEnviron();
+#endif
+
     /*
      * Synchronize the values in the environ array with the contents
      * of the Tcl "env" variable.  To do this:
@@ -341,7 +356,8 @@ TclUnsetEnv(name)
     CONST char *name;		/* Name of variable to remove (UTF-8). */
 {
     char *oldValue;
-    int length, index;
+    int length;
+    int index;
 #ifdef USE_PUTENV
     Tcl_DString envString;
     char *string;
