@@ -9,12 +9,12 @@
  *	allow scripts to be evaluated directly, without compiling.
  *
  * Copyright (c) 1997 Sun Microsystems, Inc.
- * Copyright (c) 1998 by Scriptics Corporation.
+ * Copyright (c) 1998-2000 Ajuba Solutions.
  *
  * See the file "license.terms" for information on usage and redistribution
  * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
  *
- * RCS: @(#) $Id: tclParse.c,v 1.7 1999/05/04 01:32:12 stanton Exp $
+ * RCS: @(#) $Id: tclParse.c,v 1.13.2.1 2001/04/03 22:54:38 hobbs Exp $
  */
 
 #include "tclInt.h"
@@ -260,6 +260,7 @@ Tcl_ParseCommand(interp, string, numBytes, nested, parsePtr)
     parsePtr->tokensAvailable = NUM_STATIC_TOKENS;
     parsePtr->string = string;
     parsePtr->end = string + numBytes;
+    parsePtr->term = parsePtr->end;
     parsePtr->interp = interp;
     parsePtr->incomplete = 0;
     parsePtr->errorType = TCL_PARSE_SUCCESS;
@@ -372,6 +373,7 @@ Tcl_ParseCommand(interp, string, numBytes, nested, parsePtr)
 	    break;
 	}
 	if ((type & terminators) != 0) {
+	    parsePtr->term = src;
 	    src++;
 	    break;
 	}
@@ -455,6 +457,7 @@ Tcl_ParseCommand(interp, string, numBytes, nested, parsePtr)
 	}
 
 	if ((type & terminators) != 0) {
+	    parsePtr->term = src;
 	    src++;
 	    break;
 	}
@@ -607,7 +610,7 @@ ParseTokens(src, mask, parsePtr)
 		if (nested.tokenPtr != nested.staticTokens) {
 		    ckfree((char *) nested.tokenPtr);
 		}
-		if ((src[-1] == ']') && !nested.incomplete) {
+		if ((*nested.term == ']') && !nested.incomplete) {
 		    break;
 		}
 		if (src == parsePtr->end) {
@@ -1453,7 +1456,55 @@ Tcl_EvalEx(interp, script, numBytes, flags)
 	Tcl_DecrRefCount(objv[i]);
     }
     if (gotParse) {
+	next = parse.commandStart + parse.commandSize;
+	bytesLeft -= next - p;
+	p = next;
 	Tcl_FreeParse(&parse);
+
+	if ((nested != 0) && (p > script)) {
+	    char *nextCmd = NULL;	/* pointer to start of next command */
+
+	    /*
+	     * We get here in the special case where the TCL_BRACKET_TERM
+	     * flag was set in the interpreter.
+	     *
+	     * At this point, we want to find the end of the script
+	     * (either end of script or the closing ']').
+	     */
+
+	    while ((p[-1] != ']') && bytesLeft) {
+		if (Tcl_ParseCommand(NULL, p, bytesLeft, nested, &parse)
+			!= TCL_OK) {
+		    /*
+		     * We were looking for the ']' to close the script.
+		     * But if we find a syntax error, it is ok to quit
+		     * early since in that case we no longer need to know
+		     * where the ']' is (if there was one).  We reset the
+		     * pointer to the start of the command that after the
+		     * one causing the return.  -- hobbs
+		     */
+
+		    p = (nextCmd == NULL) ? parse.commandStart : nextCmd;
+		    break;
+		}
+
+		if (nextCmd == NULL) {
+		    nextCmd = parse.commandStart;
+		}
+
+		/*
+		 * Advance to the next command in the script.
+		 */
+
+		next = parse.commandStart + parse.commandSize;
+		bytesLeft -= next - p;
+		p = next;
+		Tcl_FreeParse(&parse);
+	    }
+	    iPtr->termOffset = (p - 1) - script;
+	} else {
+	    iPtr->termOffset = p - script;
+	}    
     }
     if (objv != staticObjArray) {
 	ckfree((char *) objv);
@@ -1592,6 +1643,7 @@ Tcl_ParseVarName(interp, string, numBytes, parsePtr, append)
     unsigned char c;
     int varIndex, offset;
     Tcl_UniChar ch;
+    unsigned array;
 
     if (numBytes >= 0) {
 	end = string + numBytes;
@@ -1698,12 +1750,17 @@ Tcl_ParseVarName(interp, string, numBytes, parsePtr, append)
 	    }
 	    break;
 	}
+
+	/*
+	 * Support for empty array names here.
+	 */
+	array = ((src != end) && (*src == '('));
 	tokenPtr->size = src - tokenPtr->start;
-	if (tokenPtr->size == 0) {
+	if (tokenPtr->size == 0 && !array) {
 	    goto justADollarSign;
 	}
 	parsePtr->numTokens++;
-	if ((src != end) && (*src == '(')) {
+	if (array) {
 	    /*
 	     * This is a reference to an array element.  Call
 	     * ParseTokens recursively to parse the element name,
@@ -1952,9 +2009,44 @@ Tcl_ParseBraces(interp, string, numBytes, parsePtr, append, termPtr)
 		src += length;
 	    }
 	} else if (src == end) {
+	    int openBrace;
+
 	    if (interp != NULL) {
 		Tcl_SetResult(interp, "missing close-brace", TCL_STATIC);
 	    }
+	    /*
+	     *  Search the source string for a possible open
+	     *  brace within the context of a comment.  Since we
+	     *  aren't performing a full Tcl parse, just look for
+	     *  an open brace preceeded by a '<whitspace>#' on 
+	     *  the same line.
+	     */
+	    openBrace = 0;
+	    while (src > string ) {
+		switch (*src) {
+		    case '{': 
+			openBrace = 1; 
+			break;
+		    case '\n':
+			openBrace = 0; 
+			break;
+		    case '#':
+			if ((openBrace == 1) && (isspace(UCHAR(src[-1])))) {
+			    if (interp != NULL) {
+				Tcl_AppendResult(interp,
+					": possible unbalanced brace in comment",
+					(char *) NULL);
+			    }
+			    openBrace = -1;
+			    break;
+			}
+			break;
+		}
+		if (openBrace == -1) {
+		    break;
+		}
+		src--;
+	    }
 	    parsePtr->errorType = TCL_PARSE_MISSING_BRACE;
 	    parsePtr->term = string;
 	    parsePtr->incomplete = 1;
