@@ -3,7 +3,7 @@
 # utility procs formerly in init.tcl which can be loaded on demand
 # for package management.
 #
-# RCS: @(#) $Id: package.tcl,v 1.11 2000/02/07 22:33:17 ericm Exp $
+# RCS: @(#) $Id: package.tcl,v 1.14.2.1 2001/04/03 22:54:38 hobbs Exp $
 #
 # Copyright (c) 1991-1993 The Regents of the University of California.
 # Copyright (c) 1994-1998 Sun Microsystems, Inc.
@@ -33,13 +33,30 @@ namespace eval ::pkg {
 
 proc pkg_compareExtension { fileName {ext {}} } {
     global tcl_platform
-    if {[string length $ext] == 0} {
-	set ext [info sharedlibextension]
-    }
+    if {![string length $ext]} {set ext [info sharedlibextension]}
     if {[string equal $tcl_platform(platform) "windows"]} {
-	return [string equal -nocase [file extension $fileName] $ext]
+        return [string equal -nocase [file extension $fileName] $ext]
     } else {
-	return [string equal [file extension $fileName] $ext]
+        # Some unices add trailing numbers after the .so, so
+        # we could have something like '.so.1.2'.
+        set root $fileName
+        while {1} {
+            set currExt [file extension $root]
+            if {[string equal $currExt $ext]} {
+                return 1
+            } 
+
+	    # The current extension does not match; if it is not a numeric
+	    # value, quit, as we are only looking to ignore version number
+	    # extensions.  Otherwise we might return 1 in this case:
+	    #		pkg_compareExtension foo.so.bar .so
+	    # which should not match.
+
+	    if { ![string is integer -strict [string range $currExt 1 end]] } {
+		return 0
+	    }
+            set root [file rootname $root]
+	}
     }
 }
 
@@ -148,12 +165,22 @@ proc pkg_mkIndex {args} {
 	# Load into the child any packages currently loaded in the parent
 	# interpreter that match the -load pattern.
 
+	if {[string length $loadPat]} {
+	    if {$doVerbose} {
+		tclLog "currently loaded packages: '[info loaded]'"
+		tclLog "trying to load all packages matching $loadPat"
+	    }
+	    if {![llength [info loaded]]} {
+		tclLog "warning: no packages are currently loaded, nothing"
+		tclLog "can possibly match '$loadPat'"
+	    }
+	}
 	foreach pkg [info loaded] {
 	    if {! [string match $loadPat [lindex $pkg 1]]} {
 		continue
 	    }
-	    if {[string equal [lindex $pkg 1] "Tk"]} {
-		$c eval {set argv {-geometry +0+0}}
+	    if {$doVerbose} {
+		tclLog "package [lindex $pkg 1] matches '$loadPat'"
 	    }
 	    if {[catch {
 		load [lindex $pkg 0] [lindex $pkg 1] $c
@@ -164,6 +191,10 @@ proc pkg_mkIndex {args} {
 	    } elseif {$doVerbose} {
 		tclLog "loaded [lindex $pkg 0] [lindex $pkg 1]"
 	    }
+	    if {[string equal [lindex $pkg 1] "Tk"]} {
+		# Withdraw . if Tk was loaded, to avoid showing a window.
+		$c eval [list wm withdraw .]
+	    }
 	}
 	cd $dir
 
@@ -327,9 +358,17 @@ proc pkg_mkIndex {args} {
 		tclLog "warning: error while $what $file: $msg"
 	    }
 	} else {
+	    set what [$c eval set ::tcl::debug]
+	    if {$doVerbose} {
+		tclLog "successful $what of $file"
+	    }
 	    set type [$c eval set ::tcl::type]
 	    set cmds [lsort [$c eval array names ::tcl::newCmds]]
 	    set pkgs [$c eval set ::tcl::newPkgs]
+	    if {$doVerbose} {
+		tclLog "commands provided were $cmds"
+		tclLog "packages provided were $pkgs"
+	    }
 	    if {[llength $pkgs] > 1} {
 		tclLog "warning: \"$file\" provides more than one package ($pkgs)"
 	    }
@@ -419,7 +458,7 @@ proc tclPkgSetup {dir pkg version files} {
 # interpreter to setup the package database.
 
 proc tclMacPkgSearch {dir} {
-    foreach x [glob -nocomplain [file join $dir *.shlb]] {
+    foreach x [glob -directory $dir -nocomplain *.shlb] {
 	if {[file isfile $x]} {
 	    set res [resource open $x]
 	    foreach y [resource list TEXT $res] {
@@ -436,7 +475,8 @@ proc tclMacPkgSearch {dir} {
 # the auto_path directories and their immediate children looking for
 # pkgIndex.tcl files and sources any such files that are found to setup
 # the package database.  (On the Macintosh we also search for pkgIndex
-# TEXT resources in all files.)
+# TEXT resources in all files.)  As it searches, it will recognize changes
+# to the auto_path and scan any new directories.
 #
 # Arguments:
 # name -		Name of desired package.  Not used.
@@ -449,26 +489,36 @@ proc tclPkgUnknown {name version {exact {}}} {
     if {![info exists auto_path]} {
 	return
     }
-    for {set i [expr {[llength $auto_path] - 1}]} {$i >= 0} {incr i -1} {
+    # Cache the auto_path, because it may change while we run through
+    # the first set of pkgIndex.tcl files
+    set old_path [set use_path $auto_path]
+    while {[llength $use_path]} {
+	set dir [lindex $use_path end]
 	# we can't use glob in safe interps, so enclose the following
-	# in a catch statement
+	# in a catch statement, where we get the pkgIndex files out
+	# of the subdirectories
 	catch {
-	    foreach file [glob -nocomplain [file join [lindex $auto_path $i] \
-		    * pkgIndex.tcl]] {
+	    foreach file [glob -directory $dir -join -nocomplain \
+		    * pkgIndex.tcl] {
 		set dir [file dirname $file]
-		if {[file readable $file]} {
+		if {[file readable $file] && ![info exists procdDirs($dir)]} {
 		    if {[catch {source $file} msg]} {
 			tclLog "error reading package index file $file: $msg"
+		    } else {
+			set procdDirs($dir) 1
 		    }
 		}
 	    }
 	}
-	set dir [lindex $auto_path $i]
+	set dir [lindex $use_path end]
 	set file [file join $dir pkgIndex.tcl]
 	# safe interps usually don't have "file readable", nor stderr channel
-	if {[interp issafe] || [file readable $file]} {
+	if {([interp issafe] || [file readable $file]) && \
+		![info exists procdDirs($dir)]} {
 	    if {[catch {source $file} msg] && ![interp issafe]}  {
 		tclLog "error reading package index file $file: $msg"
+	    } else {
+		set procdDirs($dir) 1
 	    }
 	}
 	# On the Macintosh we also look in the resource fork 
@@ -476,15 +526,26 @@ proc tclPkgUnknown {name version {exact {}}} {
 	# We can't use tclMacPkgSearch in safe interps because it uses glob
 	if {(![interp issafe]) && \
 		[string equal $tcl_platform(platform) "macintosh"]} {
-	    set dir [lindex $auto_path $i]
-	    tclMacPkgSearch $dir
-	    foreach x [glob -nocomplain [file join $dir *]] {
-		if {[file isdirectory $x]} {
+	    set dir [lindex $use_path end]
+	    if {![info exists procdDirs($dir)]} {
+		tclMacPkgSearch $dir
+		set procdDirs($dir) 1
+	    }
+	    foreach x [glob -directory $dir -nocomplain *] {
+		if {[file isdirectory $x] && ![info exists procdDirs($x)]} {
 		    set dir $x
 		    tclMacPkgSearch $dir
+		    set procdDirs($dir) 1
 		}
 	    }
 	}
+	set use_path [lrange $use_path 0 end-1]
+	if {[string compare $old_path $auto_path]} {
+	    foreach dir $auto_path {
+		lappend use_path $dir
+	    }
+	    set old_path $auto_path
+	}
     }
 }
 
