@@ -8,31 +8,47 @@
  * See the file "license.terms" for information on usage and redistribution
  * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
  *
- * RCS: @(#) $Id: tclMacInt.h,v 1.5 1999/05/11 07:12:19 jingham Exp $
+ * RCS: @(#) $Id: tclMacInt.h,v 1.6.2.1 2001/04/04 21:22:19 hobbs Exp $
  */
 
 #ifndef _TCLMACINT
 #define _TCLMACINT
 
-#ifndef _TCL
-#   include "tcl.h"
+#ifndef _TCLINT
+#include "tclInt.h"
 #endif
-#ifndef _TCLMAC
-#   include "tclMac.h"
+#ifndef _TCLPORT
+#include "tclPort.h"
 #endif
 
 #include <Events.h>
 #include <Files.h>
 
-#pragma export on
-
 /*
- * Defines to control stack behavior
+ * Defines to control stack behavior.
+ *
+ * The Tcl8.2 regexp code is highly recursive for patterns with many
+ * subexpressions.  So we have to increase the stack space to accomodate.
+ * 512 K is good enough for ordinary work, but you need 768 to pass the Tcl
+ * regexp testsuite.
+ *
+ * For the PPC, you need to set the stack space in the Project file.
+ *
  */
 
-#define TCL_MAC_68K_STACK_GROWTH (256*1024)
+#ifdef TCL_TEST
+#	define TCL_MAC_68K_STACK_GROWTH (768*1024)
+#else
+#	define TCL_MAC_68K_STACK_GROWTH (512*1024)
+#endif
+
 #define TCL_MAC_STACK_THRESHOLD 16384
 
+#ifdef BUILD_tcl
+# undef TCL_STORAGE_CLASS
+# define TCL_STORAGE_CLASS DLLEXPORT
+#endif
+
 /*
  * This flag is passed to TclMacRegisterResourceFork
  * by a file (usually a library) whose resource fork
@@ -52,10 +68,9 @@
 EXTERN char *	TclMacGetFontEncoding _ANSI_ARGS_((int fontId));
 EXTERN int	TclMacHaveThreads(void);
 
-#include "tclPort.h"
-#include "tclPlatDecls.h"
+# undef TCL_STORAGE_CLASS
+# define TCL_STORAGE_CLASS DLLIMPORT
+
 #include "tclIntPlatDecls.h"
     
-#pragma export reset
-
 #endif /* _TCLMACINT */
