@@ -12,7 +12,7 @@
  * See the file "license.terms" for information on usage and redistribution
  * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
  *
- * RCS: @(#) $Id: tclMacResource.r,v 1.3 1999/04/16 00:47:21 stanton Exp $
+ * RCS: @(#) $Id: tclMacResource.r,v 1.4.2.1 2001/04/04 21:22:19 hobbs Exp $
  */
 
 #include <Types.r>
@@ -40,20 +40,6 @@
 #   define MINOR_VERSION TCL_MINOR_VERSION * 16
 #endif
 
-resource 'vers' (1) {
-	TCL_MAJOR_VERSION, MINOR_VERSION,
-	RELEASE_LEVEL, 0x00, verUS,
-	TCL_PATCH_LEVEL,
-	TCL_PATCH_LEVEL ", by Ray Johnson  Sun Microsystems"
-};
-
-resource 'vers' (2) {
-	TCL_MAJOR_VERSION, MINOR_VERSION,
-	RELEASE_LEVEL, 0x00, verUS,
-	TCL_PATCH_LEVEL,
-	"Simple Tcl Shell " TCL_PATCH_LEVEL "  1996"
-};
-
 
 /* 
  * The mechanisim below loads Tcl source into the resource fork of the
@@ -83,8 +69,27 @@ resource 'vers' (2) {
  */
  
 resource 'STR#' (128, "Tcl Environment Variables") {
-	{	"SCHEDULE_NAME=Agent Controller Schedule",
+	{	
+		/*		
+		"SCHEDULE_NAME=Agent Controller Schedule",
 		"SCHEDULE_PATH=Lozoya:System Folder:Tcl Lib:Tcl-Scheduler"
+		*/
 	};
 };
 
+data 'alis' (1000, "Library Folder") {
+	$"0000 0000 00BA 0002 0001 012F 0000 0000"            /* ........../.... */
+	$"0000 0000 0000 0000 0000 0000 0000 0000"            /* ................ */
+	$"0000 0000 0000 985C FB00 4244 0000 0000"            /* ......\.BD.... */
+	$"0002 1328 5375 7070 6F72 7420 4C69 6272"            /* ...(Support Libr */
+	$"6172 6965 7329 0000 0000 0000 0000 0000"            /* aries).......... */
+	$"0000 0000 0000 0000 0000 0000 0000 0000"            /* ................ */
+	$"0000 0000 0000 0000 0000 0000 0000 0000"            /* ................ */
+	$"0000 0076 8504 B617 A796 003D 0027 025B"            /* ...v...=.'.[ */
+	$"01E4 0001 0001 0000 0000 0000 0000 0000"            /* ............... */
+	$"0000 0000 0000 0000 0001 2F00 0002 0015"            /* ........../..... */
+	$"2F3A 2853 7570 706F 7274 204C 6962 7261"            /* /:(Support Libra */
+	$"7269 6573 2900 FFFF 0000"                           /* ries)... */
+};
+
+
