@@ -10,7 +10,7 @@
  * See the file "license.terms" for information on usage and redistribution
  * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
  *
- * RCS: @(#) $Id: tclUnixPipe.c,v 1.7 1999/12/12 02:27:20 hobbs Exp $
+ * RCS: @(#) $Id: tclUnixPipe.c,v 1.9.2.1 2001/04/03 22:54:39 hobbs Exp $
  */
 
 #include "tclInt.h"
@@ -67,16 +67,20 @@ static int	SetupStdFile _ANSI_ARGS_((TclFile file, int type));
  */
 
 static Tcl_ChannelType pipeChannelType = {
-    "pipe",				/* Type name. */
-    PipeBlockModeProc,			/* Set blocking/nonblocking mode.*/
-    PipeCloseProc,			/* Close proc. */
-    PipeInputProc,			/* Input proc. */
-    PipeOutputProc,			/* Output proc. */
-    NULL,				/* Seek proc. */
-    NULL,				/* Set option proc. */
-    NULL,				/* Get option proc. */
-    PipeWatchProc,			/* Initialize notifier. */
-    PipeGetHandleProc,			/* Get OS handles out of channel. */
+    "pipe",			/* Type name. */
+    TCL_CHANNEL_VERSION_2,	/* v2 channel */
+    PipeCloseProc,		/* Close proc. */
+    PipeInputProc,		/* Input proc. */
+    PipeOutputProc,		/* Output proc. */
+    NULL,			/* Seek proc. */
+    NULL,			/* Set option proc. */
+    NULL,			/* Get option proc. */
+    PipeWatchProc,		/* Initialize notifier. */
+    PipeGetHandleProc,		/* Get OS handles out of channel. */
+    NULL,			/* close2proc. */
+    PipeBlockModeProc,		/* Set blocking or non-blocking mode.*/
+    NULL,			/* flush proc. */
+    NULL,			/* handler proc. */
 };
 
 /*
@@ -147,7 +151,7 @@ TclpOpenFile(fname, mode)
 	 */
 
 	if (mode & O_WRONLY) {
-	    lseek(fd, 0, SEEK_END);
+	    lseek(fd, (off_t) 0, SEEK_END);
 	}
 
 	/*
@@ -182,11 +186,19 @@ TclFile
 TclpCreateTempFile(contents)
     CONST char *contents;	/* String to write into temp file, or NULL. */
 {
-    char fileName[L_tmpnam];
+    char fileName[L_tmpnam], *native;
+    Tcl_DString dstring;
     int fd;
 
-    tmpnam(fileName);					/* INTL: Native. */
-    fd = open(fileName, O_RDWR|O_CREAT|O_TRUNC, 0666);	/* INTL: Native. */
+    /*
+     * Linux says we should use mkstemp, but Solaris prefers tmpnam.
+     * We should also check against making more then TMP_MAX of these.
+     */
+
+    if (tmpnam(fileName) == NULL) {			/* INTL: Native. */
+	return NULL;
+    }
+    fd = open(fileName, O_RDWR|O_CREAT|O_EXCL, 0666);	/* INTL: Native. */
     if (fd == -1) {
 	return NULL;
     }
@@ -194,11 +206,14 @@ TclpCreateTempFile(contents)
     unlink(fileName);					/* INTL: Native. */
 
     if (contents != NULL) {
-	if (write(fd, contents, strlen(contents)) == -1) {
+	native = Tcl_UtfToExternalDString(NULL, contents, -1, &dstring);
+	if (write(fd, native, strlen(native)) == -1) {
 	    close(fd);
+	    Tcl_DStringFree(&dstring);
 	    return NULL;
 	}
-	lseek(fd, 0, SEEK_SET);
+	Tcl_DStringFree(&dstring);
+	lseek(fd, (off_t) 0, SEEK_SET);
     }
     return MakeFile(fd);
 }
@@ -438,10 +453,12 @@ TclpCreateProcess(interp, argc, argv, inputFile, outputFile, errorFile,
     if (pid != -1) {
 	/*
 	 * Reap the child process now if an error occurred during its
-	 * startup.
+	 * startup.  We don't call this with WNOHANG because that can lead to
+	 * defunct processes on an MP system.   We shouldn't have to worry
+	 * about hanging here, since this is the error case.  [Bug: 6148]
 	 */
 
-	Tcl_WaitPid((Tcl_Pid) pid, &status, WNOHANG);
+	Tcl_WaitPid((Tcl_Pid) pid, &status, 0);
     }
     
     if (errPipeIn) {
