@@ -19,7 +19,7 @@
  * See the file "license.terms" for information on usage and redistribution
  * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
  *
- * RCS: @(#) $Id: tclUnixPort.h,v 1.13 2000/01/24 02:30:17 hobbs Exp $
+ * RCS: @(#) $Id: tclUnixPort.h,v 1.15 2000/04/21 04:04:12 hobbs Exp $
  */
 
 #ifndef _TCLUNIXPORT
@@ -423,7 +423,7 @@ extern int errno;
  * Variables provided by the C library:
  */
 
-#if defined(_sgi) || defined(__sgi)
+#if defined(_sgi) || defined(__sgi) || (defined(__APPLE__) && defined(__DYNAMIC__))
 #   define environ _environ
 #endif
 extern char **environ;
@@ -461,6 +461,17 @@ extern double strtod();
 #define TclpGetPid(pid)		((unsigned long) (pid))
 #define TclpReleaseFile(file)	/* Nothing. */
 
+/*
+ * The following defines wrap the system memory allocation routines for
+ * use by tclAlloc.c.  By default off unused on Unix.
+ */
+
+#if USE_TCLALLOC
+#   define TclpSysAlloc(size, isBin)	malloc((size_t)size)
+#   define TclpSysFree(ptr)		free((char*)ptr)
+#   define TclpSysRealloc(ptr, size)	realloc((char*)ptr, (size_t)size)
+#endif
+
 /*
  * The following macros and declaration wrap the C runtime library
  * functions.
