@@ -3,12 +3,13 @@
  *
  *	Contains platform specific test commands for the Unix platform.
  *
- * Copyright (c) 1996 Sun Microsystems, Inc.
+ * Copyright (c) 1996-1997 Sun Microsystems, Inc.
+ * Copyright (c) 1998 by Scriptics Corporation.
  *
  * See the file "license.terms" for information on usage and redistribution
  * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
  *
- * SCCS: @(#) tclUnixTest.c 1.5 97/10/31 17:23:42
+ * RCS: @(#) $Id: tclUnixTest.c,v 1.11 1999/10/13 00:32:50 hobbs Exp $
  */
 
 #include "tclInt.h"
@@ -36,10 +37,6 @@
  * exercised by the "testfilehandler" command.
  */
 
-#ifdef __cplusplus
-#   define dummy /* */
-#endif
-
 typedef struct Pipe {
     TclFile readFile;		/* File handle for reading from the
 				 * pipe.  NULL means pipe doesn't exist yet. */
@@ -76,8 +73,11 @@ static int		TestfindexecutableCmd _ANSI_ARGS_((ClientData dummy,
 			    Tcl_Interp *interp, int argc, char **argv));
 static int		TestgetopenfileCmd _ANSI_ARGS_((ClientData dummy,
 			    Tcl_Interp *interp, int argc, char **argv));
+static int		TestgetdefencdirCmd _ANSI_ARGS_((ClientData dummy,
+			    Tcl_Interp *interp, int argc, char **argv));
+static int		TestsetdefencdirCmd _ANSI_ARGS_((ClientData dummy,
+			    Tcl_Interp *interp, int argc, char **argv));
 int			TclplatformtestInit _ANSI_ARGS_((Tcl_Interp *interp));
-int                   Unixtest_Init _ANSI_ARGS_((Tcl_Interp *interp));
 static int		TestalarmCmd _ANSI_ARGS_((ClientData dummy,
 			    Tcl_Interp *interp, int argc, char **argv));
 static int		TestgotsigCmd _ANSI_ARGS_((ClientData dummy,
@@ -102,49 +102,9 @@ static void 		AlarmHandler _ANSI_ARGS_(());
  */
 
 int
-#ifdef _USING_PROTOTYPES_
-TclplatformtestInit (
-    Tcl_Interp *interp)		/* Interpreter to add commands to. */
-#else
 TclplatformtestInit(interp)
     Tcl_Interp *interp;		/* Interpreter to add commands to. */
-#endif
 {
-    return Unixtest_Init(interp);
-}
-
-/*
- *----------------------------------------------------------------------
- *
- * Unixtest_Init --
- *
- *	Defines commands that test platform specific functionality for
- *	Unix platforms.
- *
- * Results:
- *	A standard Tcl result.
- *
- * Side effects:
- *	Defines new commands.
- *
- *----------------------------------------------------------------------
- */
-
-int
-#ifdef _USING_PROTOTYPES_
-Unixtest_Init (
-    Tcl_Interp *interp)		/* Interpreter to add commands to. */
-#else
-Unixtest_Init(interp)
-    Tcl_Interp *interp;		/* Interpreter to add commands to. */
-#endif
-{
-    if (Tcl_PkgRequire(interp, "Tcl", TCL_VERSION, 1) == NULL) {
-	return TCL_ERROR;
-    }
-    if (Tcl_PkgProvide(interp, "Unixtest", TCL_VERSION) != TCL_OK) {
-        return TCL_ERROR;
-    }
     Tcl_CreateCommand(interp, "testfilehandler", TestfilehandlerCmd,
             (ClientData) 0, (Tcl_CmdDeleteProc *) NULL);
     Tcl_CreateCommand(interp, "testfilewait", TestfilewaitCmd,
@@ -153,6 +113,10 @@ Unixtest_Init(interp)
             (ClientData) 0, (Tcl_CmdDeleteProc *) NULL);
     Tcl_CreateCommand(interp, "testgetopenfile", TestgetopenfileCmd,
             (ClientData) 0, (Tcl_CmdDeleteProc *) NULL);
+    Tcl_CreateCommand(interp, "testgetdefenc", TestgetdefencdirCmd,
+            (ClientData) 0, (Tcl_CmdDeleteProc *) NULL);
+    Tcl_CreateCommand(interp, "testsetdefenc", TestsetdefencdirCmd,
+            (ClientData) 0, (Tcl_CmdDeleteProc *) NULL);
     Tcl_CreateCommand(interp, "testalarm", TestalarmCmd,
             (ClientData) 0, (Tcl_CmdDeleteProc *) NULL);
     Tcl_CreateCommand(interp, "testgotsig", TestgotsigCmd,
@@ -179,19 +143,11 @@ Unixtest_Init(interp)
  */
 
 static int
-#ifdef _USING_PROTOTYPES_
-TestfilehandlerCmd (
-    ClientData dummy,			/* Not used. */
-    Tcl_Interp *interp,			/* Current interpreter. */
-    int argc,				/* Number of arguments. */
-    char **argv)			/* Argument strings. */
-#else
-TestfilehandlerCmd(dummy, interp, argc, argv)
-    ClientData dummy;			/* Not used. */
+TestfilehandlerCmd(clientData, interp, argc, argv)
+    ClientData clientData;		/* Not used. */
     Tcl_Interp *interp;			/* Current interpreter. */
     int argc;				/* Number of arguments. */
     char **argv;			/* Argument strings. */
-#endif
 {
     Pipe *pipePtr;
     int i, mask, timeout;
@@ -245,7 +201,7 @@ TestfilehandlerCmd(dummy, interp, argc, argv)
 	}
 	pipePtr->readCount = pipePtr->writeCount = 0;
     } else if (strcmp(argv[1], "counts") == 0) {
-	char buf[30];
+	char buf[TCL_INTEGER_SPACE * 2];
 	
 	if (argc != 3) {
 	    Tcl_AppendResult(interp, "wrong # arguments: should be \"",
@@ -327,7 +283,7 @@ TestfilehandlerCmd(dummy, interp, argc, argv)
             /* Empty loop body. */
         }
     } else if (strcmp(argv[1], "fillpartial") == 0) {
-	char buf[30];
+	char buf[TCL_INTEGER_SPACE];
 	
 	if (argc != 3) {
 	    Tcl_AppendResult(interp, "wrong # arguments: should be \"",
@@ -336,7 +292,7 @@ TestfilehandlerCmd(dummy, interp, argc, argv)
 	}
 
 	memset((VOID *) buffer, 'b', 10);
-	sprintf(buf, "%d", write(GetFd(pipePtr->writeFile), buffer, 10));
+	TclFormatInt(buf, write(GetFd(pipePtr->writeFile), buffer, 10));
 	Tcl_SetResult(interp, buf, TCL_VOLATILE);
     } else if (strcmp(argv[1], "oneevent") == 0) {
 	Tcl_DoOneEvent(TCL_FILE_EVENTS|TCL_DONT_WAIT);
@@ -381,18 +337,10 @@ TestfilehandlerCmd(dummy, interp, argc, argv)
     return TCL_OK;
 }
 
-static void
-#ifdef _USING_PROTOTYPES_
-TestFileHandlerProc (
-    ClientData clientData,	/* Points to a Pipe structure. */
-    int mask)			/* Indicates which events happened:
-				 * TCL_READABLE or TCL_WRITABLE. */
-#else
-TestFileHandlerProc(clientData, mask)
+static void TestFileHandlerProc(clientData, mask)
     ClientData clientData;	/* Points to a Pipe structure. */
     int mask;			/* Indicates which events happened:
 				 * TCL_READABLE or TCL_WRITABLE. */
-#endif
 {
     Pipe *pipePtr = (Pipe *) clientData;
 
@@ -498,20 +446,32 @@ TestfindexecutableCmd(clientData, interp, argc, argv)
     char **argv;			/* Argument strings. */
 {
     char *oldName;
+    char *oldNativeName;
 
     if (argc != 2) {
 	Tcl_AppendResult(interp, "wrong # arguments: should be \"", argv[0],
 		" argv0\"", (char *) NULL);
 	return TCL_ERROR;
     }
-    oldName = tclExecutableName;
-    tclExecutableName = NULL;
+
+    oldName       = tclExecutableName;
+    oldNativeName = tclNativeExecutableName;
+
+    tclExecutableName       = NULL;
+    tclNativeExecutableName = NULL;
+
     Tcl_FindExecutable(argv[1]);
     if (tclExecutableName != NULL) {
 	Tcl_SetResult(interp, tclExecutableName, TCL_VOLATILE);
 	ckfree(tclExecutableName);
     }
-    tclExecutableName = oldName;
+    if (tclNativeExecutableName != NULL) {
+	ckfree(tclNativeExecutableName);
+    }
+
+    tclExecutableName       = oldName;
+    tclNativeExecutableName = oldNativeName;
+
     return TCL_OK;
 }
 
@@ -533,19 +493,11 @@ TestfindexecutableCmd(clientData, interp, argc, argv)
  */
 
 static int
-#ifdef _USING_PROTOTYPES_
-TestgetopenfileCmd (
-    ClientData dummy,			/* Not used. */
-    Tcl_Interp *interp,			/* Current interpreter. */
-    int argc,				/* Number of arguments. */
-    char **argv)			/* Argument strings. */
-#else
-TestgetopenfileCmd(dummy, interp, argc, argv)
-    ClientData dummy;			/* Not used. */
+TestgetopenfileCmd(clientData, interp, argc, argv)
+    ClientData clientData;		/* Not used. */
     Tcl_Interp *interp;			/* Current interpreter. */
     int argc;				/* Number of arguments. */
     char **argv;			/* Argument strings. */
-#endif
 {
     ClientData filePtr;
 
@@ -568,6 +520,87 @@ TestgetopenfileCmd(dummy, interp, argc, argv)
     return TCL_OK;
 }
 
+/*
+ *----------------------------------------------------------------------
+ *
+ * TestsetdefencdirCmd --
+ *
+ *	This procedure implements the "testsetdefenc" command. It is
+ *	used to set the value of tclDefaultEncodingDir.
+ *
+ * Results:
+ *	A standard Tcl result.
+ *
+ * Side effects:
+ *	None.
+ *
+ *----------------------------------------------------------------------
+ */
+
+static int
+TestsetdefencdirCmd(clientData, interp, argc, argv)
+    ClientData clientData;		/* Not used. */
+    Tcl_Interp *interp;			/* Current interpreter. */
+    int argc;				/* Number of arguments. */
+    char **argv;			/* Argument strings. */
+{
+    if (argc != 2) {
+        Tcl_AppendResult(interp,
+                "wrong # args: should be \"", argv[0],
+                " defaultDir\"",
+                (char *) NULL);
+        return TCL_ERROR;
+    }
+
+    if (tclDefaultEncodingDir != NULL) {
+	ckfree(tclDefaultEncodingDir);
+	tclDefaultEncodingDir = NULL;
+    }
+    if (*argv[1] != '\0') {
+	tclDefaultEncodingDir = (char *)
+	    ckalloc((unsigned) strlen(argv[1]) + 1);
+	strcpy(tclDefaultEncodingDir, argv[1]);
+    }
+    return TCL_OK;
+}
+
+/*
+ *----------------------------------------------------------------------
+ *
+ * TestgetdefencdirCmd --
+ *
+ *	This procedure implements the "testgetdefenc" command. It is
+ *	used to get the value of tclDefaultEncodingDir.
+ *
+ * Results:
+ *	A standard Tcl result.
+ *
+ * Side effects:
+ *	None.
+ *
+ *----------------------------------------------------------------------
+ */
+
+static int
+TestgetdefencdirCmd(clientData, interp, argc, argv)
+    ClientData clientData;		/* Not used. */
+    Tcl_Interp *interp;			/* Current interpreter. */
+    int argc;				/* Number of arguments. */
+    char **argv;			/* Argument strings. */
+{
+    if (argc != 1) {
+        Tcl_AppendResult(interp,
+                "wrong # args: should be \"", argv[0],
+                (char *) NULL);
+        return TCL_ERROR;
+    }
+
+    if (tclDefaultEncodingDir != NULL) {
+        Tcl_AppendResult(interp, tclDefaultEncodingDir, (char *) NULL);
+    }
+    return TCL_OK;
+}
+
 /*
  *----------------------------------------------------------------------
  * TestalarmCmd --
@@ -585,7 +618,7 @@ TestgetopenfileCmd(dummy, interp, argc, argv)
  *----------------------------------------------------------------------
  */
 
-int
+static int
 TestalarmCmd(clientData, interp, argc, argv)
     ClientData clientData;		/* Not used. */
     Tcl_Interp *interp;			/* Current interpreter. */
@@ -662,7 +695,7 @@ AlarmHandler()
  *----------------------------------------------------------------------
  */
 
-int
+static int
 TestgotsigCmd(clientData, interp, argc, argv)
     ClientData clientData;		/* Not used. */
     Tcl_Interp *interp;			/* Current interpreter. */
