@@ -9,7 +9,7 @@
  * See the file "license.terms" for information on usage and redistribution
  * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
  *
- * RCS: @(#) $Id: tclWinPipe.c,v 1.9 1999/12/09 14:44:11 hobbs Exp $
+ * RCS: @(#) $Id: tclWinPipe.c,v 1.11.2.2 2001/04/03 22:54:39 hobbs Exp $
  */
 
 #include "tclWinInt.h"
@@ -124,6 +124,8 @@ typedef struct PipeInfo {
     HANDLE startReader;		/* Auto-reset event used by the main thread to
 				 * signal when the reader thread should attempt
 				 * to read from the pipe. */
+    HANDLE stopReader;		/* Manual-reset event used to alert the reader
+				 * thread to fall-out and exit */
     DWORD writeError;		/* An error caused by the last background
 				 * write.  Set to 0 if no error has been
 				 * detected.  This word is shared with the
@@ -209,7 +211,7 @@ static int		WaitForRead(PipeInfo *infoPtr, int blocking);
 
 static Tcl_ChannelType pipeChannelType = {
     "pipe",			/* Type name. */
-    PipeBlockModeProc,		/* Set blocking or non-blocking mode.*/
+    TCL_CHANNEL_VERSION_2,	/* v2 channel */
     TCL_CLOSE2PROC,		/* Close proc. */
     PipeInputProc,		/* Input proc. */
     PipeOutputProc,		/* Output proc. */
@@ -218,7 +220,10 @@ static Tcl_ChannelType pipeChannelType = {
     NULL,			/* Get option proc. */
     PipeWatchProc,		/* Set up notifier to watch the channel. */
     PipeGetHandleProc,		/* Get an OS handle from channel. */
-    PipeClose2Proc
+    PipeClose2Proc,		/* close2proc */
+    PipeBlockModeProc,		/* Set blocking or non-blocking mode.*/
+    NULL,			/* flush proc. */
+    NULL,			/* handler proc. */
 };
 
 /*
@@ -692,6 +697,8 @@ TclpCreateTempFile(contents)
     CONST char *contents;	/* String to write into temp file, or NULL. */
 {
     WCHAR name[MAX_PATH];
+    CONST char *native;
+    Tcl_DString dstring;
     HANDLE handle;
 
     if (TempFileName(name) == 0) {
@@ -712,27 +719,33 @@ TclpCreateTempFile(contents)
     if (contents != NULL) {
 	DWORD result, length;
 	CONST char *p;
+
+	/*
+	 * Convert the contents from UTF to native encoding
+	 */
+	native = Tcl_UtfToExternalDString(NULL, contents, -1, &dstring);
 	
-	for (p = contents; *p != '\0'; p++) {
+	for (p = native; *p != '\0'; p++) {
 	    if (*p == '\n') {
-		length = p - contents;
+		length = p - native;
 		if (length > 0) {
-		    if (!WriteFile(handle, contents, length, &result, NULL)) {
+		    if (!WriteFile(handle, native, length, &result, NULL)) {
 			goto error;
 		    }
 		}
 		if (!WriteFile(handle, "\r\n", 2, &result, NULL)) {
 		    goto error;
 		}
-		contents = p+1;
+		native = p+1;
 	    }
 	}
-	length = p - contents;
+	length = p - native;
 	if (length > 0) {
-	    if (!WriteFile(handle, contents, length, &result, NULL)) {
+	    if (!WriteFile(handle, native, length, &result, NULL)) {
 		goto error;
 	    }
 	}
+	Tcl_DStringFree(&dstring);
 	if (SetFilePointer(handle, 0, NULL, FILE_BEGIN) == 0xFFFFFFFF) {
 	    goto error;
 	}
@@ -741,6 +754,11 @@ TclpCreateTempFile(contents)
     return TclWinMakeFile(handle);
 
   error:
+    /* Free the native representation of the contents if necessary */
+    if (contents != NULL) {
+	Tcl_DStringFree(&dstring);
+    }
+
     TclWinConvertError(GetLastError());
     CloseHandle(handle);
     (*tclWinProcs->deleteFileProc)((TCHAR *) name);
@@ -817,7 +835,8 @@ TclpCloseFile(
 		    || ((GetStdHandle(STD_INPUT_HANDLE) != filePtr->handle)
 			    && (GetStdHandle(STD_OUTPUT_HANDLE) != filePtr->handle)
 			    && (GetStdHandle(STD_ERROR_HANDLE) != filePtr->handle))) {
-		if (CloseHandle(filePtr->handle) == FALSE) {
+		if (filePtr->handle != NULL &&
+			CloseHandle(filePtr->handle) == FALSE) {
 		    TclWinConvertError(GetLastError());
 		    ckfree((char *) filePtr);
 		    return -1;
@@ -1181,7 +1200,7 @@ TclpCreateProcess(
 
     if ((*tclWinProcs->createProcessProc)(NULL, 
 	    (TCHAR *) Tcl_DStringValue(&cmdLine), NULL, NULL, TRUE, 
-	    createFlags, NULL, NULL, &startInfo, &procInfo) == 0) {
+	    (DWORD) createFlags, NULL, NULL, &startInfo, &procInfo) == 0) {
 	TclWinConvertError(GetLastError());
 	Tcl_AppendResult(interp, "couldn't execute \"", argv[0],
 		"\": ", Tcl_PosixError(interp), (char *) NULL);
@@ -1556,6 +1575,7 @@ BuildCommandLine(
 	    Tcl_DStringAppend(&ds, "\"", 1);
 	}
     }
+    Tcl_DStringFree(linePtr);
     Tcl_WinUtfToTChar(Tcl_DStringValue(&ds), Tcl_DStringLength(&ds), linePtr);
     Tcl_DStringFree(&ds);
 }
@@ -1632,7 +1652,8 @@ TclpCreateCommandChannel(
 
 	infoPtr->readable = CreateEvent(NULL, TRUE, TRUE, NULL);
 	infoPtr->startReader = CreateEvent(NULL, FALSE, FALSE, NULL);
-	infoPtr->readThread = CreateThread(NULL, 8000, PipeReaderThread,
+	infoPtr->stopReader = CreateEvent(NULL, TRUE, FALSE, NULL);
+	infoPtr->readThread = CreateThread(NULL, 512, PipeReaderThread,
 		infoPtr, 0, &id);
 	SetThreadPriority(infoPtr->readThread, THREAD_PRIORITY_HIGHEST); 
         infoPtr->validMask |= TCL_READABLE;
@@ -1641,12 +1662,12 @@ TclpCreateCommandChannel(
     }
     if (writeFile != NULL) {
 	/*
-	 * Start the background writeer thwrite.
+	 * Start the background writer thread.
 	 */
 
 	infoPtr->writable = CreateEvent(NULL, TRUE, TRUE, NULL);
 	infoPtr->startWriter = CreateEvent(NULL, FALSE, FALSE, NULL);
-	infoPtr->writeThread = CreateThread(NULL, 8000, PipeWriterThread,
+	infoPtr->writeThread = CreateThread(NULL, 512, PipeWriterThread,
 		infoPtr, 0, &id);
 	SetThreadPriority(infoPtr->readThread, THREAD_PRIORITY_HIGHEST); 
         infoPtr->validMask |= TCL_WRITABLE;
@@ -1791,6 +1812,7 @@ PipeClose2Proc(
     int errorCode, result;
     PipeInfo *infoPtr, **nextPtrPtr;
     ThreadSpecificData *tsdPtr = TCL_TSD_INIT(&dataKey);
+    DWORD exitCode;
 
     errorCode = 0;
     if ((!flags || (flags == TCL_CLOSE_READ))
@@ -1803,29 +1825,60 @@ PipeClose2Proc(
 
 	if (pipePtr->readThread) {
 	    /*
-	     * Forcibly terminate the background thread.  We cannot rely on the
-	     * thread to cleanly terminate itself because we have no way of
-	     * closing the pipe handle without blocking in the case where the
-	     * thread is in the middle of an I/O operation.  Note that we need
-	     * to guard against terminating the thread while it is in the
-	     * middle of Tcl_ThreadAlert because it won't be able to release
-	     * the notifier lock.
+	     * Note that we need to guard against terminating the thread while
+	     * it is in the middle of Tcl_ThreadAlert because it won't be able
+	     * to release the notifier lock.
 	     */
 
 	    Tcl_MutexLock(&pipeMutex);
-	    TerminateThread(pipePtr->readThread, 0);
 
 	    /*
-	     * Wait for the thread to terminate.  This ensures that we are
-	     * completely cleaned up before we leave this function. 
+	     * The thread may already have closed on it's own.  Check it's
+	     * exit code.
 	     */
 
-	    WaitForSingleObject(pipePtr->readThread, INFINITE);
+	    GetExitCodeThread(pipePtr->readThread, &exitCode);
+
+	    if (exitCode == STILL_ACTIVE) {
+		/*
+		 * Set the stop event so that if the reader thread is blocked
+		 * in PipeReaderThread on WaitForMultipleEvents, it will exit
+		 * cleanly.
+		 */
+
+		SetEvent(pipePtr->stopReader);
+
+		/*
+		 * Wait at most 10 milliseconds for the reader thread to close.
+		 */
+
+		WaitForSingleObject(pipePtr->readThread, 10);
+		GetExitCodeThread(pipePtr->readThread, &exitCode);
+
+		if (exitCode == STILL_ACTIVE) {
+		    /*
+		     * The thread must be blocked waiting for the pipe to become
+		     * readable in ReadFile().  There isn't a clean way to exit
+		     * the thread from this condition.  We should terminate the
+		     * child process instead to get the reader thread to fall out of
+		     * ReadFile with a FALSE.  (below) is not the correct way to do
+		     * this, but will stay here until a better solution is found.
+		     */
+
+		    /* BUG: this leaks memory */
+		    TerminateThread(pipePtr->readThread, 0);
+
+		    /* Wait for the thread to terminate. */
+		    WaitForSingleObject(pipePtr->readThread, INFINITE);
+		}
+	    }
+
 	    Tcl_MutexUnlock(&pipeMutex);
 
 	    CloseHandle(pipePtr->readThread);
 	    CloseHandle(pipePtr->readable);
 	    CloseHandle(pipePtr->startReader);
+	    CloseHandle(pipePtr->stopReader);
 	    pipePtr->readThread = NULL;
 	}
 	if (TclpCloseFile(pipePtr->readFile) != 0) {
@@ -2099,9 +2152,9 @@ PipeOutputProc(
 		ckfree(infoPtr->writeBuf);
 	    }
 	    infoPtr->writeBufLen = toWrite;
-	    infoPtr->writeBuf = ckalloc(toWrite);
+	    infoPtr->writeBuf = ckalloc((unsigned int) toWrite);
 	}
-	memcpy(infoPtr->writeBuf, buf, toWrite);
+	memcpy(infoPtr->writeBuf, buf, (size_t) toWrite);
 	infoPtr->toWrite = toWrite;
 	ResetEvent(infoPtr->writable);
 	SetEvent(infoPtr->startWriter);
@@ -2344,7 +2397,7 @@ Tcl_WaitPid(
     int options)
 {
     ProcInfo *infoPtr, **prevPtrPtr;
-    int flags;
+    DWORD flags;
     Tcl_Pid result;
     DWORD ret;
 
@@ -2646,7 +2699,9 @@ WaitForRead(
  * Side effects:
  *	Signals the main thread when input become available.  May
  *	cause the main thread to wake up by posting a message.  May
- *	consume one byte from the pipe for each wait operation.
+ *	consume one byte from the pipe for each wait operation.  Will
+ *	cause a memory leak of ~4k, if forcefully terminated with
+ *	TerminateThread().
  *
  *----------------------------------------------------------------------
  */
@@ -2658,13 +2713,25 @@ PipeReaderThread(LPVOID arg)
     HANDLE *handle = ((WinFile *) infoPtr->readFile)->handle;
     DWORD count, err;
     int done = 0;
+    HANDLE wEvents[2] = {infoPtr->stopReader, infoPtr->startReader};
+    DWORD dwWait;
 
     while (!done) {
 	/*
-	 * Wait for the main thread to signal before attempting to wait.
+	 * Wait for the main thread to signal before attempting to wait
+	 * on the pipe becoming readable.
 	 */
 
-	WaitForSingleObject(infoPtr->startReader, INFINITE);
+	dwWait = WaitForMultipleObjects(2, wEvents, FALSE, INFINITE);
+
+	if (dwWait != (WAIT_OBJECT_0 + 1)) {
+	    /*
+	     * The start event was not signaled.  It might be the stop event
+	     * or an error, so exit.
+	     */
+
+	    return 0;
+	}
 
 	/*
 	 * Try waiting for 0 bytes.  This will block until some data is
